/*
 * Generated by gdbus-codegen 2.63.0 from org.freedesktop.UPower.Device.xml. DO NOT EDIT.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from.
 */

#ifndef __GF_UPOWER_DEVICE_H__
#define __GF_UPOWER_DEVICE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.UPower.Device */

#define GF_TYPE_UPOWER_DEVICE (gf_upower_device_get_type ())
#define GF_UPOWER_DEVICE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_UPOWER_DEVICE, GfUPowerDevice))
#define GF_IS_UPOWER_DEVICE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_UPOWER_DEVICE))
#define GF_UPOWER_DEVICE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GF_TYPE_UPOWER_DEVICE, GfUPowerDeviceIface))

struct _GfUPowerDevice;
typedef struct _GfUPowerDevice GfUPowerDevice;
typedef struct _GfUPowerDeviceIface GfUPowerDeviceIface;

struct _GfUPowerDeviceIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_history) (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_type,
    guint arg_timespan,
    guint arg_resolution);

  gboolean (*handle_get_statistics) (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_type);

  gboolean (*handle_refresh) (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation);

  gdouble  (*get_capacity) (GfUPowerDevice *object);

  gdouble  (*get_energy) (GfUPowerDevice *object);

  gdouble  (*get_energy_empty) (GfUPowerDevice *object);

  gdouble  (*get_energy_full) (GfUPowerDevice *object);

  gdouble  (*get_energy_full_design) (GfUPowerDevice *object);

  gdouble  (*get_energy_rate) (GfUPowerDevice *object);

  gboolean  (*get_has_history) (GfUPowerDevice *object);

  gboolean  (*get_has_statistics) (GfUPowerDevice *object);

  const gchar * (*get_icon_name) (GfUPowerDevice *object);

  gboolean  (*get_is_present) (GfUPowerDevice *object);

  gboolean  (*get_is_rechargeable) (GfUPowerDevice *object);

  gdouble  (*get_luminosity) (GfUPowerDevice *object);

  const gchar * (*get_model) (GfUPowerDevice *object);

  const gchar * (*get_native_path) (GfUPowerDevice *object);

  gboolean  (*get_online) (GfUPowerDevice *object);

  gdouble  (*get_percentage) (GfUPowerDevice *object);

  gboolean  (*get_power_supply) (GfUPowerDevice *object);

  const gchar * (*get_serial) (GfUPowerDevice *object);

  guint  (*get_state) (GfUPowerDevice *object);

  guint  (*get_technology) (GfUPowerDevice *object);

  gdouble  (*get_temperature) (GfUPowerDevice *object);

  gint64  (*get_time_to_empty) (GfUPowerDevice *object);

  gint64  (*get_time_to_full) (GfUPowerDevice *object);

  guint  (*get_type_) (GfUPowerDevice *object);

  guint64  (*get_update_time) (GfUPowerDevice *object);

  const gchar * (*get_vendor) (GfUPowerDevice *object);

  gdouble  (*get_voltage) (GfUPowerDevice *object);

  guint  (*get_warning_level) (GfUPowerDevice *object);

};

GType gf_upower_device_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gf_upower_device_interface_info (void);
guint gf_upower_device_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gf_upower_device_complete_refresh (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation);

void gf_upower_device_complete_get_history (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation,
    GVariant *data);

void gf_upower_device_complete_get_statistics (
    GfUPowerDevice *object,
    GDBusMethodInvocation *invocation,
    GVariant *data);



/* D-Bus method calls: */
void gf_upower_device_call_refresh (
    GfUPowerDevice *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gf_upower_device_call_refresh_finish (
    GfUPowerDevice *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gf_upower_device_call_refresh_sync (
    GfUPowerDevice *proxy,
    GCancellable *cancellable,
    GError **error);

void gf_upower_device_call_get_history (
    GfUPowerDevice *proxy,
    const gchar *arg_type,
    guint arg_timespan,
    guint arg_resolution,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gf_upower_device_call_get_history_finish (
    GfUPowerDevice *proxy,
    GVariant **out_data,
    GAsyncResult *res,
    GError **error);

gboolean gf_upower_device_call_get_history_sync (
    GfUPowerDevice *proxy,
    const gchar *arg_type,
    guint arg_timespan,
    guint arg_resolution,
    GVariant **out_data,
    GCancellable *cancellable,
    GError **error);

void gf_upower_device_call_get_statistics (
    GfUPowerDevice *proxy,
    const gchar *arg_type,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gf_upower_device_call_get_statistics_finish (
    GfUPowerDevice *proxy,
    GVariant **out_data,
    GAsyncResult *res,
    GError **error);

gboolean gf_upower_device_call_get_statistics_sync (
    GfUPowerDevice *proxy,
    const gchar *arg_type,
    GVariant **out_data,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *gf_upower_device_get_native_path (GfUPowerDevice *object);
gchar *gf_upower_device_dup_native_path (GfUPowerDevice *object);
void gf_upower_device_set_native_path (GfUPowerDevice *object, const gchar *value);

const gchar *gf_upower_device_get_vendor (GfUPowerDevice *object);
gchar *gf_upower_device_dup_vendor (GfUPowerDevice *object);
void gf_upower_device_set_vendor (GfUPowerDevice *object, const gchar *value);

const gchar *gf_upower_device_get_model (GfUPowerDevice *object);
gchar *gf_upower_device_dup_model (GfUPowerDevice *object);
void gf_upower_device_set_model (GfUPowerDevice *object, const gchar *value);

const gchar *gf_upower_device_get_serial (GfUPowerDevice *object);
gchar *gf_upower_device_dup_serial (GfUPowerDevice *object);
void gf_upower_device_set_serial (GfUPowerDevice *object, const gchar *value);

guint64 gf_upower_device_get_update_time (GfUPowerDevice *object);
void gf_upower_device_set_update_time (GfUPowerDevice *object, guint64 value);

guint gf_upower_device_get_type_ (GfUPowerDevice *object);
void gf_upower_device_set_type_ (GfUPowerDevice *object, guint value);

gboolean gf_upower_device_get_power_supply (GfUPowerDevice *object);
void gf_upower_device_set_power_supply (GfUPowerDevice *object, gboolean value);

gboolean gf_upower_device_get_has_history (GfUPowerDevice *object);
void gf_upower_device_set_has_history (GfUPowerDevice *object, gboolean value);

gboolean gf_upower_device_get_has_statistics (GfUPowerDevice *object);
void gf_upower_device_set_has_statistics (GfUPowerDevice *object, gboolean value);

gboolean gf_upower_device_get_online (GfUPowerDevice *object);
void gf_upower_device_set_online (GfUPowerDevice *object, gboolean value);

gdouble gf_upower_device_get_energy (GfUPowerDevice *object);
void gf_upower_device_set_energy (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_energy_empty (GfUPowerDevice *object);
void gf_upower_device_set_energy_empty (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_energy_full (GfUPowerDevice *object);
void gf_upower_device_set_energy_full (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_energy_full_design (GfUPowerDevice *object);
void gf_upower_device_set_energy_full_design (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_energy_rate (GfUPowerDevice *object);
void gf_upower_device_set_energy_rate (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_voltage (GfUPowerDevice *object);
void gf_upower_device_set_voltage (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_luminosity (GfUPowerDevice *object);
void gf_upower_device_set_luminosity (GfUPowerDevice *object, gdouble value);

gint64 gf_upower_device_get_time_to_empty (GfUPowerDevice *object);
void gf_upower_device_set_time_to_empty (GfUPowerDevice *object, gint64 value);

gint64 gf_upower_device_get_time_to_full (GfUPowerDevice *object);
void gf_upower_device_set_time_to_full (GfUPowerDevice *object, gint64 value);

gdouble gf_upower_device_get_percentage (GfUPowerDevice *object);
void gf_upower_device_set_percentage (GfUPowerDevice *object, gdouble value);

gdouble gf_upower_device_get_temperature (GfUPowerDevice *object);
void gf_upower_device_set_temperature (GfUPowerDevice *object, gdouble value);

gboolean gf_upower_device_get_is_present (GfUPowerDevice *object);
void gf_upower_device_set_is_present (GfUPowerDevice *object, gboolean value);

guint gf_upower_device_get_state (GfUPowerDevice *object);
void gf_upower_device_set_state (GfUPowerDevice *object, guint value);

gboolean gf_upower_device_get_is_rechargeable (GfUPowerDevice *object);
void gf_upower_device_set_is_rechargeable (GfUPowerDevice *object, gboolean value);

gdouble gf_upower_device_get_capacity (GfUPowerDevice *object);
void gf_upower_device_set_capacity (GfUPowerDevice *object, gdouble value);

guint gf_upower_device_get_technology (GfUPowerDevice *object);
void gf_upower_device_set_technology (GfUPowerDevice *object, guint value);

guint gf_upower_device_get_warning_level (GfUPowerDevice *object);
void gf_upower_device_set_warning_level (GfUPowerDevice *object, guint value);

const gchar *gf_upower_device_get_icon_name (GfUPowerDevice *object);
gchar *gf_upower_device_dup_icon_name (GfUPowerDevice *object);
void gf_upower_device_set_icon_name (GfUPowerDevice *object, const gchar *value);


/* ---- */

#define GF_TYPE_UPOWER_DEVICE_PROXY (gf_upower_device_proxy_get_type ())
#define GF_UPOWER_DEVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_UPOWER_DEVICE_PROXY, GfUPowerDeviceProxy))
#define GF_UPOWER_DEVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_TYPE_UPOWER_DEVICE_PROXY, GfUPowerDeviceProxyClass))
#define GF_UPOWER_DEVICE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_TYPE_UPOWER_DEVICE_PROXY, GfUPowerDeviceProxyClass))
#define GF_IS_UPOWER_DEVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_UPOWER_DEVICE_PROXY))
#define GF_IS_UPOWER_DEVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_TYPE_UPOWER_DEVICE_PROXY))

typedef struct _GfUPowerDeviceProxy GfUPowerDeviceProxy;
typedef struct _GfUPowerDeviceProxyClass GfUPowerDeviceProxyClass;
typedef struct _GfUPowerDeviceProxyPrivate GfUPowerDeviceProxyPrivate;

struct _GfUPowerDeviceProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GfUPowerDeviceProxyPrivate *priv;
};

struct _GfUPowerDeviceProxyClass
{
  GDBusProxyClass parent_class;
};

GType gf_upower_device_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfUPowerDeviceProxy, g_object_unref)
#endif

void gf_upower_device_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfUPowerDevice *gf_upower_device_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GfUPowerDevice *gf_upower_device_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gf_upower_device_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GfUPowerDevice *gf_upower_device_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GfUPowerDevice *gf_upower_device_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GF_TYPE_UPOWER_DEVICE_SKELETON (gf_upower_device_skeleton_get_type ())
#define GF_UPOWER_DEVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GF_TYPE_UPOWER_DEVICE_SKELETON, GfUPowerDeviceSkeleton))
#define GF_UPOWER_DEVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GF_TYPE_UPOWER_DEVICE_SKELETON, GfUPowerDeviceSkeletonClass))
#define GF_UPOWER_DEVICE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GF_TYPE_UPOWER_DEVICE_SKELETON, GfUPowerDeviceSkeletonClass))
#define GF_IS_UPOWER_DEVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GF_TYPE_UPOWER_DEVICE_SKELETON))
#define GF_IS_UPOWER_DEVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GF_TYPE_UPOWER_DEVICE_SKELETON))

typedef struct _GfUPowerDeviceSkeleton GfUPowerDeviceSkeleton;
typedef struct _GfUPowerDeviceSkeletonClass GfUPowerDeviceSkeletonClass;
typedef struct _GfUPowerDeviceSkeletonPrivate GfUPowerDeviceSkeletonPrivate;

struct _GfUPowerDeviceSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GfUPowerDeviceSkeletonPrivate *priv;
};

struct _GfUPowerDeviceSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gf_upower_device_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GfUPowerDeviceSkeleton, g_object_unref)
#endif

GfUPowerDevice *gf_upower_device_skeleton_new (void);


G_END_DECLS

#endif /* __GF_UPOWER_DEVICE_H__ */
